/************************************************

  WindowSizer.c

  A simple utility that will make a given window
  a specific size. Useful for web developers to
  test web sites as they will look in different
  resolutions.

  by Mike Lin

  Permission is granted to use this source as
  you please.

************************************************/

#include <windows.h>
#include "resource.h"

/*
FillWindowsListProc

EnumWindows proc to fill windows list
*/

BOOL CALLBACK FillWindowsListProc(HWND hwnd, LPARAM lParam)
{
	char wndtxt[1024];						// buffer for window text
	HWND hwndWindowsList = (HWND) lParam;	// HWND of windows list box

	if(hwndWindowsList == NULL)
		return FALSE;

	if(!IsWindowVisible(hwnd))				// continue if window is invisible
		return TRUE;

	if(GetWindowText(hwnd, wndtxt, 1024) == 0)	// continue if window has no text
		return TRUE;

	if(!strcmp(wndtxt,"Program Manager"))
		return TRUE;

	if(!strcmp(wndtxt,"WindowSizer by Mike Lin"))
		return TRUE;

	SendMessage(hwndWindowsList, LB_ADDSTRING, 0, (WPARAM) wndtxt);	// add window
	
	return TRUE;
}

/*
FillWindowsList

Fills windows list box with visible top-level windows (EnumWindows)
*/

BOOL FillWindowsList(HWND hwndWindowsList)
{
	return EnumWindows(FillWindowsListProc, (LPARAM) hwndWindowsList);
}

/*
SizeWindow

Sizes the selected window to the size specified by the user
*/

BOOL SizeWindow(HWND hwndDlg)
{
	HWND hwnd;
	HWND hwndWindowsList;
	char szcx[5], szcy[5], szWndTxt[1024];
	int cx,cy, nCurSel;

	hwndWindowsList = GetDlgItem(hwndDlg, IDC_WINDOWS_LIST);

	if(hwndWindowsList == NULL)
		return FALSE;

	// Get current selection

	if((nCurSel = SendMessage(hwndWindowsList, LB_GETCURSEL, 0, 0)) == LB_ERR)
	{
		MessageBox(hwndDlg, "Please select a window whose size you want to set.", "Error", MB_OK);
		return FALSE;
	}

	// Get current selection text

	if(SendMessage(hwndWindowsList, LB_GETTEXT, (WPARAM) nCurSel, (LPARAM) szWndTxt) == LB_ERR)
		return FALSE;

	// Find window based on caption

	hwnd = FindWindow(NULL, szWndTxt);

	if(hwnd == NULL)
	{
		MessageBox(hwndDlg, "Couldn't find window!", "Error", MB_OK);
		return FALSE;
	}

	// Get sizes in text

	if( (GetWindowText(GetDlgItem(hwndDlg, IDC_X), szcx, 5) == 0) || 
		(GetWindowText(GetDlgItem(hwndDlg, IDC_Y), szcy, 5) == 0) )
	{
		MessageBox(hwndDlg, "Please specify the size to set the window.", "Error", MB_OK);
		return FALSE;
	}

	// Convert sizes to numbers

	cx = atoi(szcx);
	cy = atoi(szcy);

	if((cx == 0) || (cy == 0))
	{
		MessageBox(hwndDlg, "Invalid size", "Error", MB_OK);
		return FALSE;
	}

	// Set window size

	SetWindowPos(hwnd, NULL, 0, 0, cx, cy, SWP_NOMOVE | SWP_NOZORDER);

	return TRUE;
}

/*
UpdateSizes

Updates the size text boxes when the selection is changed
*/

BOOL UpdateSizes(HWND hwndDlg)
{
	HWND hwnd,hwndWindowsList,hwndX,hwndY;
	char sztxt[5];
	char szwndtxt[1024];
	int nCurSel;
	RECT rcWnd;

	hwndWindowsList = GetDlgItem(hwndDlg, IDC_WINDOWS_LIST);
	hwndX = GetDlgItem(hwndDlg, IDC_X);
	hwndY = GetDlgItem(hwndDlg, IDC_Y);

	if((hwndWindowsList == NULL) || (hwndX == NULL) || (hwndY == NULL))
		return FALSE;

	nCurSel = SendMessage(hwndWindowsList, LB_GETCURSEL, 0, 0);

	if(nCurSel == LB_ERR)
		return FALSE;

	SendMessage(hwndWindowsList, LB_GETTEXT, (WPARAM) nCurSel, (LPARAM) szwndtxt);

	hwnd = FindWindow(NULL, szwndtxt);

	if((hwnd == NULL) || (!IsWindow(hwnd)))
		return FALSE;

	if(!GetWindowRect(hwnd, &rcWnd))
		return FALSE;

	wsprintf(sztxt, "%d", (rcWnd.right - rcWnd.left));

	SetWindowText(hwndX, sztxt);

	wsprintf(sztxt, "%d", (rcWnd.bottom - rcWnd.top));

	SetWindowText(hwndY, sztxt);
	
	return TRUE;
}

/*
OnInitDialog

WM_INITDIALOG handler
*/

BOOL OnInitDialog(HWND hwnd)
{
	HWND hwndWindowsList;

	hwndWindowsList = GetDlgItem(hwnd, IDC_WINDOWS_LIST);

	if(hwndWindowsList == NULL)
		return FALSE;

	if(!FillWindowsList(hwndWindowsList))
		return FALSE;

	return TRUE;
}

/*
OnCommand

WM_COMMAND handler
*/

BOOL OnCommand(HWND hwnd, WORD wNotifyCode, WORD wID, HWND hwndCtl)
{
	switch(wID)
	{
	case IDC_EXIT:
		return EndDialog(hwnd, 0);
	case IDC_SET:
		return SizeWindow(hwnd);
	case IDC_REFRESH:
		SendMessage(GetDlgItem(hwnd, IDC_WINDOWS_LIST), LB_RESETCONTENT, 0, 0);
		FillWindowsList(GetDlgItem(hwnd, IDC_WINDOWS_LIST));
		return TRUE;
	case IDC_WINDOWS_LIST:
		if(wNotifyCode == LBN_SELCHANGE)
			return UpdateSizes(hwnd);
		return FALSE;
	}

	return FALSE;
}

BOOL CALLBACK DlgProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
	case WM_INITDIALOG:
		return OnInitDialog(hwnd);
	case WM_COMMAND:
		return OnCommand(hwnd, HIWORD(wParam), LOWORD(wParam), (HWND) lParam);
	case WM_CLOSE:
		return EndDialog(hwnd, 0);
	}

	return FALSE;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	DialogBox(hInstance,
			  MAKEINTRESOURCE(IDD_WINDOWSIZER),
			  NULL,
			  DlgProc);

	return 0;
}